<?php

namespace Drupal\grouper\Controller;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;


/**
 * Class GrouperController.
 */
class GrouperController extends ControllerBase {

  /**
   * Drupal\Core\Database\Driver\mysql\Connection definition.
   *
   * @var \Drupal\Core\Database\Driver\mysql\Connection
   */
  protected $database;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->database = $container->get('database');
    return $instance;
  }

  /**
   * Shows the most frequent log messages of a given event type.
   *
   * Messages are not truncated on this page because events detailed herein do
   * not have links to a detailed view.
   *
   * @return array
   *   A build array in the format expected by
   *   \Drupal\Core\Render\RendererInterface::render().
   */
  public function summary() {
    $connection = \Drupal::database();
    $connection->query("SET SQL_MODE=''");

    $header = [
      ['data' => $this->t('Count'), 'field' => 'count', 'sort' => 'desc'],
      ['data' => $this->t('Type'), 'field' => 'type'],
      ['data' => $this->t('Message'), 'field' => 'message'],
    ];

    $count_query = $connection->select('watchdog');
    $count_query->addExpression('COUNT(DISTINCT(message))');

    $query = $connection->select('watchdog', 'w')
      ->extend('\Drupal\Core\Database\Query\PagerSelectExtender')
      ->extend('\Drupal\Core\Database\Query\TableSortExtender');
    $query->addExpression('COUNT(wid)', 'count');

    $query->fields('w', ['message', 'variables', 'type', 'wid']);

    // Base Condition exclude php messages.
    $query->condition('w.type', 'php', '!=');

    if (isset($_GET['type'])) {
      $query->condition('w.type', $_GET['type']);
    }

    if (isset($_GET['severity'])) {
      $query->condition('w.severity', $_GET['severity']);
    }

    $query->groupBy('message');
    $query->groupBy('variables');
    $query->limit(30);
    $query->orderByHeader($header);
    $query->setCountQuery($count_query);
    $result = $query->execute();

    $rows = [];
    foreach ($result as $dblog) {
      if ($message = $this->formatMessage($dblog)) {
        $rows[] = [$this->formatQuantity($dblog), $dblog->type, $message];
      }
    }

    $build['pre-table'] = [
      '#children' => $this->renderFilterForm(),
    ];

    $build['dblog_top_table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No log messages available.'),
      '#attached' => [
        'library' => ['dblog/drupal.dblog'],
      ],
    ];
    $build['dblog_top_pager'] = ['#type' => 'pager'];

    return $build;
  }

  /**
   * Formats a quantity message.
   *
   * @param object $row
   *   The record from the watchdog table.
   *   The object properties are: message, variables, count.
   */
  public function formatQuantity($row) {
    $wid = $row->wid;
    $target_path = "/admin/reports/grouper/detail/$wid/";

    $link = "<a href=\"$target_path\" class=\"nid-link\" target=\"_blank\">$row->count</a>";
    return ['data' => ['#markup' => $link]];
  }

  /**
   * Formats a database log message.
   *
   * @param object $row
   *   The record from the watchdog table. The object properties are: wid, uid,
   *   severity, type, timestamp, message, variables, link, name.
   *
   * @return string|\Drupal\Core\StringTranslation\TranslatableMarkup|false
   *   The formatted log message or FALSE if the message or variables properties
   *   are not set.
   */
  public function formatMessage($row) {
    // Check for required properties.
    if (isset($row->message, $row->variables)) {
      $variables = @unserialize($row->variables);
      // Messages without variables or user specified text.
      if ($variables === NULL) {
        $message = Xss::filterAdmin($row->message);
      }
      elseif (!is_array($variables)) {
        $message = $this->t('Log data is corrupted and cannot be unserialized: @message', ['@message' => Xss::filterAdmin($row->message)]);
      }
      // Message to translate with injected variables.
      else {
        $message = $this->t(Xss::filterAdmin($row->message), $variables);
      }
    }
    else {
      $message = FALSE;
    }
    return $message;
  }

  /**
   * Renders the Filters Form.
   *
   * @return string
   *   Return Error Type select list markup.
   */
  public function renderFilterForm() {
    $form_markup = '<form action="" METHOD="GET">';

    $form_markup .= '<table style="max-width: 400px;">';
    $form_markup .= '<tr><td>' . $this->renderTypeSelect() . '</td>';
    $form_markup .= '<td>' . $this->renderSeveritySelect() . '</td>';
    $form_markup .= '<td valign="bottom"><input id="filter-submit" type="submit" value="Filter"></td>';
    $form_markup .= '<td valign="bottom"><input type="reset" value="reset"></td><tr>';
    $form_markup .= '</table>';
    $form_markup .= '</form>';
    return $form_markup;
  }

  /**
   * Error Type Selects.
   *
   * @return string
   *   Return Error Type select list markup.
   */
  public function renderTypeSelect() {

    $database = \Drupal::database();
    $query = $database->query("SELECT DISTINCT `type` FROM {watchdog} WHERE `type` != 'php'");
    $result = $query->fetchAll();

    $markup = '<label for="select-type">Type</label>
             <select data-drupal-selector="edit-severity"
                     name="type"
                     id="select-type"
                     size="8"
                     class="form-select">';

    foreach ($result as $typeObj) {
      $markup .= "<option value=\"$typeObj->type\">$typeObj->type</option>";
    }

    $markup .= '</select></span>';
    return $markup;
  }

  /**
   * Severity Selects.
   *
   * @return string
   *   Return Severity Select List Markup.
   */
  public function renderSeveritySelect() {
    return '<label for="select-severity">Severity</label>
          <select data-drupal-selector="edit-severity"
                  name="severity"
                  id="select-severity"
                  size="8"
                  class="form-select">
                    <option value="0">Emergency</option>
                    <option value="1">Alert</option>
                    <option value="2">Critical</option>
                    <option value="3">Error</option>
                    <option value="4">Warning</option>
                    <option value="5">Notice</option>
                    <option value="6">Info</option>
                    <option value="7">Debug</option>
          </select>';
  }

  /**
   * Detail.
   *
   * @return string
   *   Return Hello string.
   */
  public function detail($wid) {

    $database = \Drupal::database();

    // The Quantity.
    $qty = $this->database->query("SELECT COUNT(*) FROM {watchdog} WHERE `variables` = (SELECT `variables` FROM {watchdog} WHERE `wid` = :wid)", [':wid' => $wid])->fetchField();

    // The Message.
    $row = $this->database->query("SELECT `message`, `variables`, `type` FROM {watchdog} WHERE `wid` = :wid", [':wid' => $wid])->fetchAll();

    $type = ($row[0]->type ?? '');

    $quantity = '<div id="php-quantity-type">' . $qty . '&nbsp;&nbsp;' . $type . '</div>';

    $msg = ($this->formatMessage($row[0]) ?? '');
    $message = '<div id="php-message"><pre>' . $msg . '</pre></div>';

    $header = [
                 ['data' => $this->t('Date')],
                 ['data' => $this->t('Message')],
    ];

    $rows = [];

    // Other Detail Items.
    $result2 = $this->database->query("SELECT DISTINCT `wid`, `timestamp`, `location`, `referer` FROM {watchdog} WHERE `variables` = (SELECT `variables` FROM {watchdog} WHERE `wid` = :wid)", [':wid' => $wid]);

    foreach ($result2 as $row) {
      $rows[] = [
        'data' => [
          // Cells.
          $this->formatDateWidLink($row),
          $this->formatLinkAndReferrer($row),
        ],
        // Attributes for table row.
        'class' => ['grouper-row'],
      ];

    }

    $build['quantity'] = [
      '#children' => $quantity,
    ];

    $build['message'] = [
      '#children' => $message,
    ];

    $build['dblog_top_table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No log messages available.'),
      '#attached' => [
        'library' => ['dblog/drupal.dblog'],
      ],
    ];

    $build['dblog_top_pager'] = ['#type' => 'pager'];

    return $build;

  }

  /**
   * Formats a The Event Tima and Event Link.
   *
   * @param object $row
   *   The record from the watchdog table. The object properties are: wid, uid,
   *   severity, type, timestamp, message, variables, link, name.
   *
   * @return string|\Drupal\Core\StringTranslation\TranslatableMarkup|false
   *   The formatted log message or FALSE if the message or variables properties
   *   are not set.
   */
  public function formatDateWidLink($row) {
    $link_text = date("d-m-Y H:i:s", $row->timestamp);
    $target_path = "/admin/reports/dblog/event/$row->wid/";
    $title = 'Click To View Event';

    $link = "<a href=\"$target_path\" class=\"wid-link\" title=\"$title\" target=\"_blank\">$link_text</a>";
    return ['data' => ['#markup' => $link]];
  }

  /**
   * Formats a Link and referrer.
   *
   * @param object $row
   *   The record from the watchdog table. The object properties are: wid, uid,
   *   severity, type, timestamp, message, variables, link, name.
   *
   * @return string|\Drupal\Core\StringTranslation\TranslatableMarkup|false
   *   The formatted log message or FALSE if the message or variables properties
   *   are not set.
   */
  public function formatLinkAndReferrer($row) {
    $link = "<a href=\"$row->location\" class=\"wid-link\" title=\"REFERRER $row->referer\" target=\"_blank\">$row->location</a>";
    return ['data' => ['#markup' => $link]];
  }

}
