<?php

namespace Drupal\grouper\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\Unicode;
use Drupal\Component\Utility\Xss;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class GrouperPhpController.
 */
class GrouperPhpController extends ControllerBase {

  /**
   * Drupal\Core\Database\Driver\mysql\Connection definition.
   *
   * @var \Drupal\Core\Database\Driver\mysql\Connection
   */
  protected $database;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->database = $container->get('database');
    return $instance;
  }

  /**
   * Summary.
   *
   * @return string
   *   Returns Table.
   */
  public function summary() {

    $this->database->query("SET SQL_MODE=''");

    $type = 'php';

    $header = [
            ['data' => $this->t('Count'), 'field' => 'count', 'sort' => 'desc'],
            ['data' => $this->t('Message'), 'field' => 'message'],
    ];

    $count_query = $this->database->select('watchdog');
    $count_query->addExpression('COUNT(DISTINCT(message))');
    $count_query->condition('type', $type);

    $query = $this->database->select('watchdog', 'w')
      ->extend('\Drupal\Core\Database\Query\PagerSelectExtender')
      ->extend('\Drupal\Core\Database\Query\TableSortExtender');
    $query->addExpression('COUNT(wid)', 'count');
    $query = $query
      ->fields('w', ['message', 'variables', 'wid'])
      ->condition('w.type', $type)
      ->groupBy('message')
      ->groupBy('variables')
      ->limit(30)
      ->orderByHeader($header);
    $query->setCountQuery($count_query);
    $result = $query->execute();

    $rows = [];

    foreach ($result as $dblog) {
      $quantity = $this->formatQuantity($dblog);
      $message = $this->formatMessage($dblog);

      $rows[] = [
        'data' => [
          // Cells.
          $quantity,
          $message,
        ],
        // Attributes for table row.
        'class' => ['grouper-row'],
      ];
    }

    $build['dblog_top_table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No log messages available.'),
      '#attached' => [
        'library' => ['dblog/drupal.dblog'],
      ],
    ];
    $build['dblog_top_pager'] = ['#type' => 'pager'];

    return $build;

  }

  /**
   * Formats a quantity message.
   *
   * @param object $row
   *   The record from the watchdog table.
   *   The object properties are: message, variables, count.
   */
  public function formatQuantity($row) {
    $wid = $row->wid;
    $target_path = "/admin/reports/grouper/php-detail/$wid/";

    $link = "<a href=\"$target_path\" class=\"nid-link\" target=\"_blank\">$row->count</a>";
    return ['data' => ['#markup' => $link]];
  }

  /**
   * Formats a database log message.
   *
   * @param object $row
   *   The record from the watchdog table. The object properties are: wid, uid,
   *   severity, type, timestamp, message, variables, link, name.
   *
   * @return string|\Drupal\Core\StringTranslation\TranslatableMarkup|false
   *   The formatted log message or FALSE if the message or variables properties
   *   are not set.
   */
  public function formatMessage($row) {
    // Check for required properties.
    if (isset($row->message, $row->variables)) {
      $variables = @unserialize($row->variables);
      // Messages without variables or user specified text.
      if ($variables === NULL) {
        $message = Xss::filterAdmin($row->message);
      }
      elseif (!is_array($variables)) {
        $message = $this->t('Log data is corrupted and cannot be unserialized: @message', ['@message' => Xss::filterAdmin($row->message)]);
      }
      // Message to translate with injected variables.
      else {
        $rawBacktraceString = $variables['@backtrace_string'];
        $backtraceString = $this->formatBacktrace($rawBacktraceString);

        // Format the message so its on 3 lines.
        if ((strcmp($row->message, '%type: @message in %function (line %line of %file).') == 0) ||
            (strcmp($row->message, '%type: @message in %function (line %line of %file) @backtrace_string.') == 0)) {
          // Put it on 3 lines.
          $message_template = '%type: @message <br/>in %function <br/>(line %line of %file).';
        }

        $message_text = $this->t(Xss::filterAdmin($message_template), $variables);

        $div = "<div title=\"$backtraceString\">$message_text</div>";
        $message = ['data' => ['#markup' => $div]];

      }

    }
    else {
      $message = FALSE;
    }

    return $message;
  }

  /**
   * Formats a backtrace string.
   */
  public function formatBacktrace($rawBacktraceString) {
    $massaged1 = Unicode::truncate(Html::decodeEntities(strip_tags($rawBacktraceString)), strlen($rawBacktraceString), TRUE, TRUE);
    $massaged2 = str_replace('"', '', $massaged1);
    $massaged3 = str_replace('->', "-&gt;", $massaged2);
    $massaged4 = str_replace('#', "\r\n#", $massaged3);
    return $massaged4;
  }

  /**
   * Detail.
   *
   * @return string
   *   Return Hello string.
   */
  public function detail($wid) {

    // The Quantity.
    $qty = $this->database->query("SELECT COUNT(*) FROM {watchdog} WHERE `variables` = (SELECT `variables` FROM {watchdog} WHERE `wid` = :wid)", [':wid' => $wid])->fetchField();
    $quantity = '<div id="php-quantity">' . $qty . '</div>';

    // The Message.
    $row = $this->database->query("SELECT `message`, `variables` FROM {watchdog} WHERE `wid` = :wid", [':wid' => $wid])->fetchAll();
    $msg = ($this->formatMessage($row[0])['data']['#markup'] ?? '');
    $message = '<div id="php-message"><pre>' . $msg . '</pre></div>';

    // The Backtrace.
    $variables = @unserialize($row[0]->variables);
    $rawBacktraceString = $variables['@backtrace_string'];
    $backtraceString = $this->formatBacktrace($rawBacktraceString);
    $backtrce = '<div id="php-backtrace"><details open><summary>PHP Backtrace</summary><pre>' . $backtraceString . '</pre></details></div>';

    $header = [
                 ['data' => $this->t('Date')],
                 ['data' => $this->t('Message')],
    ];

    $rows = [];

    // Other Detail Items.
    $result2 = $this->database->query("SELECT DISTINCT `wid`, `timestamp`, `location`, `referer` FROM {watchdog} WHERE `variables` = (SELECT `variables` FROM {watchdog} WHERE `wid` = :wid)", [':wid' => $wid]);
    foreach ($result2 as $row) {
      $rows[] = [
        'data' => [
          // Cells.
          $this->formatDateWidLink($row),
          $this->formatLinkAndReferrer($row),
        ],
        // Attributes for table row.
        'class' => ['grouper-row'],
      ];

    }

    $build['quantity'] = [
      '#children' => $quantity,
    ];

    $build['message'] = [
      '#children' => $message,
    ];

    $build['backtrace'] = [
      '#children' => $backtrce,
    ];

    $build['dblog_top_table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No log messages available.'),
      '#attached' => [
        'library' => ['dblog/drupal.dblog'],
      ],
    ];
    // A $build['dblog_top_pager'] = ['#type' => 'pager']; .
    // A.
    return $build;

  }

  /**
   * Formats a The Event Tima and Event Link.
   *
   * @param object $row
   *   The record from the watchdog table. The object properties are: wid, uid,
   *   severity, type, timestamp, message, variables, link, name.
   *
   * @return string|\Drupal\Core\StringTranslation\TranslatableMarkup|false
   *   The formatted log message or FALSE if the message or variables properties
   *   are not set.
   */
  public function formatDateWidLink($row) {
    $link_text = date("d-m-Y H:i:s", $row->timestamp);
    $target_path = "/admin/reports/dblog/event/$row->wid/";
    $title = 'Click To View Event';

    $link = "<a href=\"$target_path\" class=\"wid-link\" title=\"$title\" target=\"_blank\">$link_text</a>";
    return ['data' => ['#markup' => $link]];
  }

  /**
   * Formats a Link and referrer.
   *
   * @param object $row
   *   The record from the watchdog table. The object properties are: wid, uid,
   *   severity, type, timestamp, message, variables, link, name.
   *
   * @return string|\Drupal\Core\StringTranslation\TranslatableMarkup|false
   *   The formatted log message or FALSE if the message or variables properties
   *   are not set.
   */
  public function formatLinkAndReferrer($row) {
    $link = "<a href=\"$row->location\" class=\"wid-link\" title=\"REFERRER $row->referer\" target=\"_blank\">$row->location</a>";
    return ['data' => ['#markup' => $link]];
  }

}
